@@ -7,6 +7,7 @@ by Itxaka Serrano Garcia <itxakaserrano@gmail.com>
 import requests
 import json
 import markdown
+from . import exceptions
 
 
 class Gitlab(object):
@@ -26,10 +27,15 @@ class Gitlab(object):
             self.host = host[:-1]
         else:
             self.host = host
-        self.projects_url = self.host + "/api/v3/projects"
-        self.users_url = self.host + "/api/v3/users"
-        self.keys_url = self.host + "/api/v3/user/keys"
-        self.groups_url = self.host + "/api/v3/groups"
+        if self.host[:7] != 'http://':
+            self.host = 'http://' + self.host
+
+        self.api_url = self.host + "/api/v3"
+        self.projects_url = self.api_url + "/projects"
+        self.users_url = self.api_url + "/users"
+        self.keys_url = self.api_url + "/user/keys"
+        self.groups_url = self.api_url + "/groups"
+        self.search_url = self.api_url + "/projects/search/"
         self.verify_ssl = verify_ssl
 
     def login(self, user, password):
@@ -49,7 +55,7 @@ class Gitlab(object):
                             "connection": "close"}
             return True
         else:
-            return False
+            raise exceptions.HttpError(json.loads(request.content)['message'])
 
     def getusers(self, page=1, per_page=20):
         """
@@ -66,6 +72,19 @@ class Gitlab(object):
         else:
             return False
 
+    def getuser(self, id_):
+        """
+        Get info for a user identified by id
+        :param id_: id of the user
+        :return: False if not found, a dictionary if found
+        """
+        request = requests.get(self.users_url + "/" + str(id_),
+                               headers=self.headers, verify=self.verify_ssl)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
     def createuser(self, name,
                    username,
                    password,
@@ -289,7 +308,24 @@ class Gitlab(object):
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
         else:
-            
+            return False
+
+    def getallprojects(self, page=1, per_page=20, sudo=""):
+        """
+        Returns a dictionary of all the projects for admins only
+        :param page: Which page to return (default is 1)
+        :param per_page: Number of items to return per page (default is 20)
+        :return: list with the repo name, description, last activity,
+         web url, ssh url, owner and if its public
+        """
+        data = {'page': page, 'per_page': per_page}
+        if sudo != "":
+            data['sudo'] = sudo
+        request = requests.get(self.projects_url + '/all', params=data,
+                               headers=self.headers, verify=self.verify_ssl)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
             return False
 
     def getproject(self, id_):
@@ -325,7 +361,7 @@ class Gitlab(object):
             
             return False
 
-    def createproject(self, name, description="", default_branch="",
+    def createproject(self, name, description="",
                       issues_enabled=0, wall_enabled=0,
                       merge_requests_enabled=0, wiki_enabled=0,
                       snippets_enabled=0, public=0, sudo=""):
@@ -336,7 +372,6 @@ class Gitlab(object):
          False otherwise
         """
         data = {"name": name, "description": description,
-                "default_branch": default_branch,
                 "issues_enabled": issues_enabled, "wall_enabled": wall_enabled,
                 "merge_requests_enabled": merge_requests_enabled,
                 "wiki_enabled": wiki_enabled,
@@ -362,6 +397,18 @@ class Gitlab(object):
             
             return False
 
+    def deleteproject(self, project_id):
+        """
+        Delete a project
+        :param id_: project id
+        :return: always true
+        """
+        request = requests.delete(self.projects_url + "/" + str(project_id),
+                                  headers=self.headers)
+        if request.status_code == 200:
+            return True
+
+
     def createprojectuser(self, id_, name, description="", default_branch="",
                           issues_enabled=0, wall_enabled=0,
                           merge_requests_enabled=0, wiki_enabled=0,
@@ -472,6 +519,9 @@ class Gitlab(object):
         if request.status_code == 200:
             return True  # It always returns true
 
+    # TODO: hooks are now system wide and under the /hooks url
+    # TODO: change all the hooks methods, not valid anymore
+    # TODO: write tests for the hooks
     def getprojecthooks(self, id_):
         """
         get all the hooks from a project
@@ -941,11 +991,12 @@ class Gitlab(object):
         request = requests.post(self.groups_url,
                                 data={'name': name, 'path': path},
                                 headers=self.headers, verify=self.verify_ssl)
+        print request.status_code
+        print request.content
         if request.status_code == 201:
             return True
         else:
-            
-            return False
+            return exceptions.HttpError(json.loads(request.content)['message'])
 
     def getgroups(self, id_=None, page=1, per_page=20, sudo=""):
         """
@@ -1246,7 +1297,7 @@ class Gitlab(object):
             data['ref_name'] = ref_name
 
         request = requests.get(self.projects_url + "/" + str(project_id) +
-                               "/repository/tree/", data=data,
+                               "/repository/tree/", params=data,
                                headers=self.headers)
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
@@ -1258,8 +1309,353 @@ class Gitlab(object):
 
         request = requests.get(self.projects_url + "/" + str(project_id) +
                                "/repository/blobs/" + str(sha1),
-                               data=data, headers=self.headers)
+                               params=data, headers=self.headers)
         if request.status_code == 200:
             return request.content.decode("utf-8")
         else:
             return False
+
+    def searchproject(self, search, page=1, per_page=20):
+        """
+        projects section
+        """
+        data = {'page': page, 'per_page': per_page}
+        request = requests.get(self.search_url + str(search), params=data,
+                               headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            raise exceptions.HttpError(json.loads(request.content)['message'])
+
+    def getfilearchive(self, project_id, filepath="", sha1=""):
+        """
+        repository section
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) +
+                               "/repository/archive", headers=self.headers)
+        if request.status_code == 200:
+            if filepath == "":
+                filepath = request.headers['content-disposition'].split(";")[1].split("=")[1].strip('"')
+            with open(filepath, "wb") as filesave:
+                filesave.write(request.content)
+                # TODO: Catch oserror exceptions as no permissions and such
+                # TODO: change the filepath to a path and keep always the filename?
+            return True
+        else:
+            raise exceptions.HttpError(json.loads(request.content)['message'])
+
+    def deletegroup(self, group_id):
+        """
+        groups section, new in 6.2
+
+        Deletes an group by ID
+        :param id_: id of the group to delete
+        :return: True if it deleted, False if it couldn't. False could happen
+        for several reasons, but there isn't a
+        good way of differentiating them
+        """
+        request = requests.delete(self.groups_url + "/" + str(group_id),
+                                  headers=self.headers)
+        if request.status_code == 200:
+            return True
+        else:
+            return False
+
+    def listgroupmembers(self, group_id):
+        """
+        list group members
+        new in 6.2
+
+        lists the members of a given group id
+        :param group_id: the group id
+        :return: the group's members
+        """
+        request = requests.get(self.groups_url + "/" + str(group_id) + "/members",
+                               headers=self.headers, verify=self.verify_ssl)
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def addgroupmember(self, group_id, user_id, access_level, sudo=""):
+        """
+        add a user to a group
+        new in 6.2
+
+        # check the access level and put into a number
+
+        adds a project member to a project
+        :param id_: project id
+        :param user_id: user id
+        :param access_level: access level, see gitlab help to know more
+        :param sudo: do the request with another user
+        :return: True if success
+        """
+        if access_level.lower() == "master":
+            access_level = 40
+        elif access_level.lower() == "developer":
+            access_level = 30
+        elif access_level.lower() == "reporter":
+            access_level = 20
+        else:
+            access_level = 10
+        data = {"id": group_id, "user_id": user_id, "access_level": access_level}
+        if sudo != "":
+            data['sudo'] = sudo
+        request = requests.post(self.groups_url + "/" + str(group_id) + "/members",
+                                headers=self.headers, data=data, verify=self.verify_ssl)
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def deletegroupmember(self, group_id, user_id):
+        """
+        Delete a group member
+        :param id_: project id
+        :param user_id: user id
+        :return: always true
+        """
+        request = requests.delete(self.groups_url + "/" + str(group_id)
+                                  + "/members/" + str(user_id),
+                                  headers=self.headers)
+        if request.status_code == 200:
+            return True  # It always returns true
+
+    def getprojectwallnotes(self, project_id):
+        """
+        get the notes from the wall of a project
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/notes",
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getprojectwallnote(self, project_id, note_id):
+        """
+        get one note from the wall of the project
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/notes/" + str(note_id),
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def createprojectwallnote(self, project_id, content):
+        """
+        create a new note
+        """
+        data = {"body": content}
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/notes",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def getissuewallnotes(self, project_id, issue_id):
+        """
+        get the notes from the wall of a issue
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/issues/" + str(issue_id) + "/notes",
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getissuewallnote(self, project_id, issue_id, note_id):
+        """
+        get one note from the wall of the issue
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/issues/" + str(issue_id) + "/notes/" + str(note_id),
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def createissuewallnote(self, project_id, issue_id, content):
+        """
+        create a new note
+        """
+        data = {"body": content}
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/issues/" + str(issue_id) + "/notes",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def getsnippetwallnotes(self, project_id, snippet_id):
+        """
+        get the notes from the wall of a snippet
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/snippets/" + str(snippet_id) + "/notes",
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getsnippetwallnote(self, project_id, snippet_id, note_id):
+        """
+        get one note from the wall of the snippet
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/snippets/" + str(snippet_id) + "/notes/" + str(note_id),
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def createsnippetewallnote(self, project_id, snippet_id, content):
+        """
+        create a new note
+        """
+        data = {"body": content}
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/snippets/" + str(snippet_id) + "/notes",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def getmergerequestwallnotes(self, project_id, merge_request_id):
+        """
+        get the notes from the wall of a merge request
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/merge_requests/" + str(merge_request_id) + "/notes",
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def getmergerequestwallnote(self, project_id, merge_request_id, note_id):
+        """
+        get one note from the wall of the merge request
+        """
+        request = requests.get(self.projects_url + "/" + str(project_id) + "/merge_requests/" + str(merge_request_id) + "/notes/" + str(note_id),
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return json.loads(request.content.decode("utf-8"))
+        else:
+            return False
+
+    def createmergerequestewallnote(self, project_id, merge_request_id, content):
+        """
+        create a new note
+        """
+        data = {"body": content}
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/merge_requests/" + str(merge_request_id) + "/notes",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def createfile(self, project_id, file_path, branch_name, content, commit_message):
+        """
+        Creates a new file in the repository
+        :param project_id: project id
+        :param file_path: Full path to new file. Ex. lib/class.rb
+        :param branch_name: The name of branch
+        :param content: File content
+        :param commit_message: Commit message
+        :return: true if success, false if not
+        """
+        data = {"file_path": file_path, "branch_name": branch_name,
+                "content": content, "commit_message": commit_message}
+        request = requests.post(self.projects_url + "/" + str(project_id) + "/repository/files",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 201:
+            return True
+        else:
+            return False
+
+    def updatefile(self, project_id, file_path, branch_name, content, commit_message):
+        """
+        Updates an existing file in the repository
+        :param project_id: project id
+        :param file_path: Full path to new file. Ex. lib/class.rb
+        :param branch_name: The name of branch
+        :param content: File content
+        :param commit_message: Commit message
+        :return: true if success, false if not
+        """
+        data = {"file_path": file_path, "branch_name": branch_name,
+                "content": content, "commit_message": commit_message}
+        request = requests.put(self.projects_url + "/" + str(project_id) + "/repository/files",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 200:
+            return True
+        else:
+            return False
+
+    def deletefile(self, project_id, file_path, branch_name, commit_message):
+        """
+        Deletes existing file in the repository
+        :param project_id: project id
+        :param file_path: Full path to new file. Ex. lib/class.rb
+        :param branch_name: The name of branch
+        :param commit_message: Commit message
+        :return: true if success, false if not
+        """
+        data = {"file_path": file_path, "branch_name": branch_name,
+                "commit_message": commit_message}
+        request = requests.delete(self.projects_url + "/" + str(project_id) + "/repository/files",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 200:
+            return True
+        else:
+            return False
+
+    def setgitlabciservice(self, project_id, token, project_url):
+        """
+        Set GitLab CI service for project
+        :param project_id: project id
+        :param token: CI project token
+        :param project_url: CI project url
+        :return: true if success, false if not
+        """
+        data = {"token": token, "project_url": project_url}
+        request = requests.put(self.projects_url + "/" + str(project_id) + "/services/gitlab-ci",
+                                  headers=self.headers, data=data)
+
+        if request.status_code == 200:
+            return True
+        else:
+            return False
+
+    def deletegitlabciservice(self, project_id, token, project_url):
+        """
+        Delete GitLab CI service settings
+        :return: true if success, false if not
+        """
+        request = requests.delete(self.projects_url + "/" + str(project_id) + "/services/gitlab-ci",
+                                  headers=self.headers)
+
+        if request.status_code == 200:
+            return True
+        else:
+            return False
\ No newline at end of file
